import requests


def test_ipv4(httpserver_ipv4):
    httpserver_ipv4.expect_request("/").respond_with_data("OK")
    assert httpserver_ipv4.host == "127.0.0.1"

    response = requests.get(httpserver_ipv4.url_for("/"))
    assert response.text == "OK"


def test_ipv6(httpserver_ipv6):
    httpserver_ipv6.expect_request("/").respond_with_data("OK")
    assert httpserver_ipv6.host == "::1"
    assert httpserver_ipv6.url_for("/") == f"http://[::1]:{httpserver_ipv6.port}/"

    response = requests.get(httpserver_ipv6.url_for("/"))
    assert response.text == "OK"



from typing import Optional

import requests

from pytest_httpserver import HTTPServer


def case_insensitive_matcher(header_name: str, actual: Optional[str], expected: str) -> bool:
    if actual is None:
        return False

    if header_name == "X-Foo":
        return actual.lower() == expected.lower()
    else:
        return actual == expected


def test_case_insensitive_matching(httpserver: HTTPServer):
    httpserver.expect_request(
        "/", header_value_matcher=case_insensitive_matcher, headers={"X-Foo": "bar"}
    ).respond_with_data("OK")

    assert requests.get(httpserver.url_for("/"), headers={"X-Foo": "bar"}).status_code == 200
    assert requests.get(httpserver.url_for("/"), headers={"X-Foo": "BAR"}).status_code == 200



import pytest
import requests


def test_connection_refused():
    # assumes that there's no server listening at localhost:1234
    with pytest.raises(requests.exceptions.ConnectionError):
        requests.get("http://localhost:1234")



import os

import pytest

from pytest_httpserver import HTTPServer
from pytest_httpserver.pytest_plugin import get_httpserver_listen_address

PORT_KEY = "PYTEST_HTTPSERVER_PORT"
HOST_KEY = "PYTEST_HTTPSERVER_HOST"


@pytest.fixture
def tmpenv():
    old_vars = {}
    for key in (HOST_KEY, PORT_KEY):
        old_vars[key] = os.environ.get(key)

    os.environ[HOST_KEY] = "5.5.5.5"
    os.environ[PORT_KEY] = "12345"

    yield

    for key, value in old_vars.items():
        if value:
            os.environ[key] = value
        else:
            del os.environ[key]


@pytest.mark.skipif(HOST_KEY not in os.environ, reason="requires {} environment variable".format(HOST_KEY))
def test_host_changing_by_environment(httpserver: HTTPServer):
    assert httpserver.host == os.environ[HOST_KEY]


@pytest.mark.skipif(PORT_KEY not in os.environ, reason="requires {} environment variable".format(PORT_KEY))
def test_port_changing_by_environment(httpserver: HTTPServer):
    assert httpserver.port == int(os.environ[PORT_KEY])


def test_get_httpserver_listen_address_with_env(tmpenv):
    address = get_httpserver_listen_address()
    assert address[0] == "5.5.5.5"
    assert address[1] == 12345



import requests


def test_query_params(httpserver):
    httpserver.expect_request("/foo", query_string={"user": "user1", "group": "group1"}).respond_with_data("OK")

    assert requests.get(httpserver.url_for("/foo?user=user1&group=group1")).status_code == 200
    assert requests.get(httpserver.url_for("/foo?group=group1&user=user1")).status_code == 200



import urllib.parse
from typing import List
from typing import Tuple

import pytest
import werkzeug.urls
from werkzeug.datastructures import MultiDict

parse_qsl_semicolon_cases = [
    ("&", []),
    ("&&", []),
    ("&a=b", [("a", "b")]),
    ("a=a+b&b=b+c", [("a", "a b"), ("b", "b c")]),
    ("a=1&a=2", [("a", "1"), ("a", "2")]),
    ("a=", [("a", "")]),
    ("a=foo bar&b=bar foo", [("a", "foo bar"), ("b", "bar foo")]),
    ("a=foo%20bar&b=bar%20foo", [("a", "foo bar"), ("b", "bar foo")]),
    ("a=%20%21%22%23%24%25%26%27%28%29%2A%2B%2C%2F%3A%3B%3D%3F%40%5B%5D", [("a", " !\"#$%&'()*+,/:;=?@[]")]),
]


@pytest.mark.parametrize("qs,expected", parse_qsl_semicolon_cases)
def test_qsl(qs: str, expected: List[Tuple[bytes, bytes]]):
    assert urllib.parse.parse_qsl(qs, keep_blank_values=True) == expected


@pytest.mark.skip(reason="skipped to avoid werkzeug warnings")
@pytest.mark.parametrize("qs,expected", parse_qsl_semicolon_cases)
def test_qsl_werkzeug(qs: str, expected: List[Tuple[bytes, bytes]]):
    assert werkzeug.urls.url_decode(qs) == MultiDict(expected)



from pytest_httpserver import HTTPServer
from pytest_httpserver import URIPattern


class PrefixMatch(URIPattern):
    def __init__(self, prefix: str):
        self.prefix = prefix

    def match(self, uri):
        return uri.startswith(self.prefix)


def test_uripattern_object(httpserver: HTTPServer):
    httpserver.expect_request(PrefixMatch("/foo")).respond_with_json({"foo": "bar"})



from typing import Optional

import requests

from pytest_httpserver import HeaderValueMatcher
from pytest_httpserver import HTTPServer


def case_insensitive_compare(actual: Optional[str], expected: str) -> bool:
    # actual is `None` if it is not specified
    if actual is None:
        return False
    return actual.lower() == expected.lower()


def test_own_matcher_object(httpserver: HTTPServer):
    matcher = HeaderValueMatcher({"X-Bar": case_insensitive_compare})

    httpserver.expect_request("/", headers={"X-Bar": "bar"}, header_value_matcher=matcher).respond_with_data("OK")

    assert requests.get(httpserver.url_for("/"), headers={"X-Bar": "bar"}).status_code == 200
    assert requests.get(httpserver.url_for("/"), headers={"X-Bar": "BAR"}).status_code == 200



import pytest
import requests

from pytest_httpserver import HTTPServer


def test_check_assertions_raises_handler_assertions(httpserver: HTTPServer):
    def handler(_):
        assert 1 == 2

    httpserver.expect_request("/foobar").respond_with_handler(handler)

    requests.get(httpserver.url_for("/foobar"))

    # if you leave this "with" statement out, check_assertions() will break
    # the test by re-raising the assertion error caused by the handler
    # pytest will pick this exception as it was happened in the main thread
    with pytest.raises(AssertionError):
        httpserver.check_assertions()

    httpserver.check_handler_errors()


def test_check_handler_errors_raises_handler_error(httpserver: HTTPServer):
    def handler(_):
        raise ValueError("should be propagated")

    httpserver.expect_request("/foobar").respond_with_handler(handler)

    requests.get(httpserver.url_for("/foobar"))

    httpserver.check_assertions()

    # if you leave this "with" statement out, check_handler_errors() will
    # break the test with the original exception
    with pytest.raises(ValueError):
        httpserver.check_handler_errors()



import requests

from pytest_httpserver import HTTPServer


def test_json_matcher(httpserver: HTTPServer):
    httpserver.expect_request("/foo", json={"foo": "bar"}).respond_with_data("Hello world!")
    resp = requests.get(httpserver.url_for("/foo"), json={"foo": "bar"})
    assert resp.status_code == 200
    assert resp.text == "Hello world!"



import http.client

import requests
from werkzeug.datastructures import Headers
from werkzeug.http import parse_dict_header

from pytest_httpserver import HTTPServer
from pytest_httpserver.httpserver import HeaderValueMatcher


def test_custom_headers(httpserver: HTTPServer):
    headers_with_values_in_direct_order = {"Custom": 'Scheme key0="value0", key1="value1"'}
    httpserver.expect_request(uri="/", headers=headers_with_values_in_direct_order).respond_with_data("OK")
    response = requests.get(httpserver.url_for("/"), headers=headers_with_values_in_direct_order)
    assert response.status_code == 200
    assert response.text == "OK"

    # By default different order of items in header value dicts means different header values
    headers_with_values_in_modified_order = {"Custom": 'Scheme key1="value1", key0="value0"'}
    response = requests.get(httpserver.url_for("/"), headers=headers_with_values_in_modified_order)
    assert response.status_code == 500

    # Define header_value_matcher that ignores the order of items in header value dict
    def custom_header_value_matcher(actual: str, expected: str) -> bool:
        actual_scheme, _, actual_dict_str = actual.partition(" ")
        expected_scheme, _, expected_dict_str = expected.partition(" ")
        actual_dict = parse_dict_header(actual_dict_str)
        expected_dict = parse_dict_header(expected_dict_str)
        return actual_scheme == expected_scheme and actual_dict == expected_dict

    matchers = HeaderValueMatcher.DEFAULT_MATCHERS.copy()  # type: ignore
    matchers["Custom"] = custom_header_value_matcher
    header_value_matcher = HeaderValueMatcher(matchers)

    httpserver.handlers.clear()
    httpserver.expect_request(
        uri="/", headers=headers_with_values_in_direct_order, header_value_matcher=header_value_matcher
    ).respond_with_data("OK")
    response = requests.get(httpserver.url_for("/"), headers=headers_with_values_in_modified_order)
    assert response.status_code == 200
    assert response.text == "OK"


# See https://en.wikipedia.org/wiki/Digest_access_authentication
def test_authorization_headers(httpserver: HTTPServer):
    headers_with_values_in_direct_order = {
        "Authorization": (
            'Digest username="Mufasa",'
            'realm="testrealm@host.com",'
            'nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",'
            'uri="/dir/index.html",'
            "qop=auth,"
            "nc=00000001,"
            'cnonce="0a4f113b",'
            'response="6629fae49393a05397450978507c4ef1",'
            'opaque="5ccc069c403ebaf9f0171e9517f40e41"'
        )
    }
    httpserver.expect_request(uri="/", headers=headers_with_values_in_direct_order).respond_with_data("OK")
    response = requests.get(httpserver.url_for("/"), headers=headers_with_values_in_direct_order)
    assert response.status_code == 200
    assert response.text == "OK"

    headers_with_values_in_modified_order = {
        "Authorization": (
            "Digest qop=auth,"
            'username="Mufasa",'
            'nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",'
            'uri="/dir/index.html",'
            "nc=00000001,"
            'realm="testrealm@host.com",'
            'response="6629fae49393a05397450978507c4ef1",'
            'cnonce="0a4f113b",'
            'opaque="5ccc069c403ebaf9f0171e9517f40e41"'
        )
    }
    response = requests.get(httpserver.url_for("/"), headers=headers_with_values_in_modified_order)
    assert response.status_code == 200
    assert response.text == "OK"


def test_header_one_key_multiple_values(httpserver: HTTPServer):
    httpserver.expect_request(uri="/t1").respond_with_data(headers=[("X-Foo", "123"), ("X-Foo", "456")])
    httpserver.expect_request(uri="/t2").respond_with_data(headers={"X-Foo": ["123", "456"]})

    headers = Headers()
    headers.add("X-Foo", "123")
    headers.add("X-Foo", "456")

    httpserver.expect_request(uri="/t3").respond_with_data(headers=headers)

    for uri in ("/t1", "/t2", "/t3"):
        conn = http.client.HTTPConnection("localhost:{}".format(httpserver.port))
        conn.request("GET", uri)
        response = conn.getresponse()
        conn.close()

        assert response.status == 200
        assert response.headers.get_all("X-Foo") == ["123", "456"]



import pytest
import requests

from pytest_httpserver import HTTPServer


@pytest.mark.xfail
def test_check_assertions(httpserver: HTTPServer):
    def handler(_):
        assert 1 == 2

    httpserver.expect_request("/foobar").respond_with_handler(handler)

    requests.get(httpserver.url_for("/foobar"))

    # this will raise AssertionError:
    httpserver.check()



import requests

from pytest_httpserver import HTTPServer


def test_wait_success(httpserver: HTTPServer):
    waiting_timeout = 0.1

    with httpserver.wait(stop_on_nohandler=False, timeout=waiting_timeout) as waiting:
        requests.get(httpserver.url_for("/foobar"))
        httpserver.expect_oneshot_request("/foobar").respond_with_data("OK foobar")
        requests.get(httpserver.url_for("/foobar"))
    assert waiting.result

    httpserver.expect_oneshot_request("/foobar").respond_with_data("OK foobar")
    httpserver.expect_oneshot_request("/foobaz").respond_with_data("OK foobaz")
    with httpserver.wait(timeout=waiting_timeout) as waiting:
        requests.get(httpserver.url_for("/foobar"))
        requests.get(httpserver.url_for("/foobaz"))
    assert waiting.result



import os
import ssl
from os.path import join as pjoin

import pytest
import requests

pytestmark = pytest.mark.ssl

test_dir = os.path.dirname(os.path.realpath(__file__))
assets_dir = pjoin(test_dir, "assets")


@pytest.fixture(scope="session")
def httpserver_ssl_context():
    protocol = None
    for name in ("PROTOCOL_TLS_SERVER", "PROTOCOL_TLS", "PROTOCOL_TLSv1_2"):
        if hasattr(ssl, name):
            protocol = getattr(ssl, name)
            break

    assert protocol is not None, "Unable to obtain TLS protocol"

    return ssl.SSLContext(protocol)


def test_ssl(httpserver):
    server_crt = pjoin(assets_dir, "server.crt")
    server_key = pjoin(assets_dir, "server.key")
    root_ca = pjoin(assets_dir, "rootCA.crt")
    context = httpserver.ssl_context

    assert (
        context is not None
    ), "SSLContext not set. The session was probably started with a test that did not define an SSLContext."

    httpserver.ssl_context.load_cert_chain(server_crt, server_key)
    httpserver.expect_request("/foobar").respond_with_json({"foo": "bar"})

    assert httpserver.is_running()
    assert requests.get(httpserver.url_for("/foobar"), verify=root_ca).json() == {"foo": "bar"}



import os

import pytest

from .httpserver import HTTPServer


class Plugin:
    SERVER = None


class PluginHTTPServer(HTTPServer):
    def start(self):
        super().start()
        Plugin.SERVER = self

    def stop(self):
        super().stop()
        Plugin.SERVER = None


def get_httpserver_listen_address():
    listen_host = os.environ.get("PYTEST_HTTPSERVER_HOST")
    listen_port = os.environ.get("PYTEST_HTTPSERVER_PORT")
    if listen_port:
        listen_port = int(listen_port)

    return listen_host, listen_port


@pytest.fixture(scope="session")
def httpserver_listen_address():
    return get_httpserver_listen_address()


@pytest.fixture(scope="session")
def httpserver_ssl_context():
    return None


@pytest.fixture(scope="session")
def make_httpserver(httpserver_listen_address, httpserver_ssl_context):
    host, port = httpserver_listen_address
    if not host:
        host = HTTPServer.DEFAULT_LISTEN_HOST
    if not port:
        port = HTTPServer.DEFAULT_LISTEN_PORT

    server = HTTPServer(host=host, port=port, ssl_context=httpserver_ssl_context)
    server.start()
    yield server
    server.clear()
    if server.is_running():
        server.stop()


def pytest_sessionfinish(session, exitstatus):  # pylint: disable=unused-argument
    if Plugin.SERVER is not None:
        Plugin.SERVER.clear()
        if Plugin.SERVER.is_running():
            Plugin.SERVER.stop()


@pytest.fixture
def httpserver(make_httpserver):
    server = make_httpserver
    yield server
    server.clear()


@pytest.fixture(scope="session")
def make_httpserver_ipv4(httpserver_ssl_context):
    server = HTTPServer(host="127.0.0.1", port=0, ssl_context=httpserver_ssl_context)
    server.start()
    yield server
    server.clear()
    if server.is_running():
        server.stop()


@pytest.fixture
def httpserver_ipv4(make_httpserver_ipv4):
    server = make_httpserver_ipv4
    yield server
    server.clear()


@pytest.fixture(scope="session")
def make_httpserver_ipv6(httpserver_ssl_context):
    server = HTTPServer(host="::1", port=0, ssl_context=httpserver_ssl_context)
    server.start()
    yield server
    server.clear()
    if server.is_running():
        server.stop()


@pytest.fixture
def httpserver_ipv6(make_httpserver_ipv6):
    server = make_httpserver_ipv6
    yield server
    server.clear()



import requests

from pytest_httpserver import HTTPServer


def test_authorization_headers(httpserver: HTTPServer):
    headers_with_values_in_direct_order = {
        "Authorization": (
            'Digest username="Mufasa",'
            'realm="testrealm@host.com",'
            'nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",'
            'uri="/dir/index.html",'
            "qop=auth,"
            "nc=00000001,"
            'cnonce="0a4f113b",'
            'response="6629fae49393a05397450978507c4ef1",'
            'opaque="5ccc069c403ebaf9f0171e9517f40e41"'
        )
    }
    httpserver.expect_request(uri="/", headers=headers_with_values_in_direct_order).respond_with_data("OK")
    response = requests.get(httpserver.url_for("/"), headers=headers_with_values_in_direct_order)
    assert response.status_code == 200
    assert response.text == "OK"

    headers_with_values_in_modified_order = {
        "Authorization": (
            "Digest qop=auth,"
            'username="Mufasa",'
            'nonce="dcd98b7102dd2f0e8b11d0f600bfb0c093",'
            'uri="/dir/index.html",'
            "nc=00000001,"
            'realm="testrealm@host.com",'
            'response="6629fae49393a05397450978507c4ef1",'
            'cnonce="0a4f113b",'
            'opaque="5ccc069c403ebaf9f0171e9517f40e41"'
        )
    }
    response = requests.get(httpserver.url_for("/"), headers=headers_with_values_in_modified_order)
    assert response.status_code == 200
    assert response.text == "OK"



import pytest
import requests

from pytest_httpserver import HTTPServer


def _setup_oneshot(server: HTTPServer):
    server.expect_request("/permanent").respond_with_data("OK permanent")
    server.expect_oneshot_request("/oneshot1").respond_with_data("OK oneshot1")
    server.expect_oneshot_request("/oneshot2").respond_with_data("OK oneshot2")


def _setup_ordered(server: HTTPServer):
    server.expect_ordered_request("/ordered1").respond_with_data("OK ordered1")
    server.expect_ordered_request("/ordered2").respond_with_data("OK ordered2")


def _setup_all(server: HTTPServer):
    _setup_oneshot(server)
    _setup_ordered(server)


def test_oneshot_and_permanent_happy_path1(httpserver: HTTPServer):
    _setup_oneshot(httpserver)
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"
    assert requests.get(httpserver.url_for("/oneshot1")).text == "OK oneshot1"
    assert requests.get(httpserver.url_for("/oneshot2")).text == "OK oneshot2"
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"

    assert len(httpserver.oneshot_handlers) == 0


def test_oneshot_and_permanent_happy_path2(httpserver: HTTPServer):
    _setup_oneshot(httpserver)
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"
    assert requests.get(httpserver.url_for("/oneshot2")).text == "OK oneshot2"
    assert requests.get(httpserver.url_for("/oneshot1")).text == "OK oneshot1"
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"

    assert len(httpserver.oneshot_handlers) == 0


def test_all_happy_path1(httpserver: HTTPServer):
    _setup_all(httpserver)

    # ordered must go first
    assert requests.get(httpserver.url_for("/ordered1")).text == "OK ordered1"
    assert requests.get(httpserver.url_for("/ordered2")).text == "OK ordered2"
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"
    assert requests.get(httpserver.url_for("/oneshot2")).text == "OK oneshot2"
    assert requests.get(httpserver.url_for("/oneshot1")).text == "OK oneshot1"
    assert requests.get(httpserver.url_for("/permanent")).text == "OK permanent"

    assert len(httpserver.oneshot_handlers) == 0
    assert len(httpserver.ordered_handlers) == 0


def test_all_ordered_missing(httpserver: HTTPServer):
    _setup_all(httpserver)

    # ordered is missing so everything must fail
    # a.k.a. permanently fail

    requests.get(httpserver.url_for("/permanent"))
    with pytest.raises(AssertionError):
        httpserver.check_assertions()

    requests.get(httpserver.url_for("/oneshot2"))
    with pytest.raises(AssertionError):
        httpserver.check_assertions()

    requests.get(httpserver.url_for("/oneshot1"))
    with pytest.raises(AssertionError):
        httpserver.check_assertions()

    requests.get(httpserver.url_for("/permanent"))
    with pytest.raises(AssertionError):
        httpserver.check_assertions()

    # handlers must be still intact but as the ordered are failed
    # everything will fail
    assert len(httpserver.ordered_handlers) == 2
    assert len(httpserver.oneshot_handlers) == 2
    assert len(httpserver.handlers) == 1


def test_repr(httpserver: HTTPServer):
    assert repr(httpserver) == f"<HTTPServer host=localhost port={httpserver.port}>"



from random import randint

from werkzeug.wrappers import Request
from werkzeug.wrappers import Response

from pytest_httpserver import HTTPServer


def test_expected_request_handler(httpserver: HTTPServer):
    def handler(request: Request):
        return Response(str(randint(1, 10)))

    httpserver.expect_request("/foobar").respond_with_handler(handler)
